/**
 * Copyright 2019 - MagabeLab (Tanzania). All Rights Reserved.
 * Author Edwin Magabe    edyma50@yahoo.com
 */

import 'package:petitparser/petitparser.dart';
import 'package:simple_smart_scraper/src/parsers/simple_flatten_parser.dart';

import 'parsers.dart';

/// Parse text except children elements
class ElementTextParser extends Parser<String> {
  String tag;
  ElementTextParser(this.tag);

  @override
  Parser<String> copy() {
    return ElementTextParser(tag);
  }

  Parser<String> flatten([String message]) {
    return SimpleFlattenParser(this, message);
  }

  @override
  Result<String> parseOn(Context context) {
    final position = context.position;
    final buffer = context.buffer;

    if (position < buffer.length) {
      final input = buffer.substring(position);
      var result = parsers.getParserResult(
          parser: parsers.qualifiedElement(tag), input: input);
      if (result.isEmpty) {
        return Failure<String>(
            buffer, position, 'No element with $tag tag found.');
      }
      final index = buffer.indexOf(result, position);
      if (index == -1) {
        return Failure<String>(buffer, position,
            'Failed to parse, No element with $tag tag found.');
      }
      final newPosition = index + result.length;

      var value;
      try {
        value = parsers.strip(result);
        final isText = (!parsers.childrenElements().accept(value));
        if (isText) {
          return Success<String>(buffer, newPosition, value);
        } else {
          final children = parsers.getParserResults(
              parser: parsers.childrenElements(), input: value);
          final childrenCount = children.length;
          if (childrenCount == 1) {
            final isClosed = parsers
                .elementStartTag(isClosed: true)
                .accept(children.elementAt(0));
            if (isClosed) {
              return Failure<String>(buffer, newPosition,
                  '$tag does not contain text, but a closed element \n>>> ${children.elementAt(0)} <<<');
            } else {
              return Failure<String>(buffer, newPosition,
                  '$tag does not contain text, but an element \n>>> ${children.elementAt(0)} <<<');
            }
          } else if (childrenCount > 1) {
            return Failure<String>(buffer, newPosition,
                '$tag does not contain text, but $childrenCount elements \n>>> $children <<<');
          } else {
            return Failure<String>(
                buffer, newPosition, '$tag does not contain any text');
          }
        }
      } on StripException catch (_) {
        final isClosed =
            parsers.elementStartTag(isClosed: true).accept(_.output);
        if (isClosed) {
          return Failure<String>(buffer, newPosition,
              '$tag is a closed element, therefore does not contain text \n>>> ${_.output} <<<');
        } else {
          return Failure<String>(buffer, newPosition, '$tag striping failed.');
        }
      }
    } else {
      return Failure<String>(buffer, buffer.length,
          'Failed to get text, position > buffer.length.');
    }
  }
}
